﻿; ScreenSaver.ahk[UTF-8] (2024/03/31 08:00)
; for AutoHotKey v2.0.12.0 or later
;
; リモート接続されるサーバ側ではディスプレイの電源をOFFにしたりスクリーンセーバーを
; 起動したりといった事が抑止されています。そのため自前の判定によってセーバーを起動
; するものです。
;
; 使い方
;  各種パラメータはiniファイルに記述します
;
; マウスが動かない時間が一定時間以上になったらスクリーンセーバーを起動します。
; 途中でマウスが動くと中断し、また一定時間操作されないと動作を再開します。
;
; クリックやキー入力は無視されマウスの移動だけで判定しています。
;
; マウスカーソルが左下にある時以外は起動しません。
; 設定したウィンドウがアクティブな時以外は起動しません。
;

/* タスクトレイアイコンにメニュー項目追加 */
A_TrayMenu.Insert("1&", "ExecForce after 5sec", handleMenuItem)

/* 変数初期化 */
Path := Array()
IniFile := "ScreenSaver.ini"
IniSect := "APP"
noDialog    := Number(IniRead(IniFile, IniSect, "NoDialog", "0"))
pastMin     := Number(IniRead(IniFile, IniSect, "SaverTime", "30"))
saverPath   := IniRead(IniFile, IniSect, "SaverPath", "C:\Windows\SysWOW64\scrnsave.scr")
ExecForce   := Number(IniRead(IniFile, IniSect, "ExecForce", "0"))
enableKey   := Number(IniRead(IniFile, IniSect, "EnableKey", "0"))
X_LIMIT     := Number(IniRead(IniFile, IniSect, "X_LIMIT", "500"))
Y_LIMIT     := Number(IniRead(IniFile, IniSect, "Y_LIMIT", "600"))
lastTime    := A_Now
lastX       := -1
lastY       := -1
modeActive  := 0 ;// 1=マウスが動いてる, 0=一定時間以上止まっている

;iniファイルから件数を読み込む
count := IniRead(IniFile, IniSect, "MAX", 0)
if( count = 0 ){
	MsgBox("iniファイルに定義がありません")
	ExitApp()
}
idx := 1
while( count > 0 ){
	;フルパスの読み込み(PATH1～)
	strTemp := IniRead(IniFile, IniSect, "PATH" String(idx), "")
	Path.Push( strTemp )
	count := count - 1
	idx := idx + 1
}


;//MouseGetPosはスクリーンを対象とする
CoordMode("Mouse", "Screen")

if(not noDialog){
	MsgBox("スクリーンセーバーの常駐を開始します",,"0x40040 T5")
}

/* 座標を獲得して前回値と比較する */
Loop{
	try {
		MouseGetPos( &posX, &posY, &posWin, &posCtrl )
		if( posX != lastX or posY != lastY ){
			lastX := posX
			lastY := posY
			lastTime := A_Now
			modeActive := 1
		}
		past := DateDiff( A_Now, lastTime, "Seconds" )
		if( (past > pastMin * 60) and (posX < X_LIMIT) and (posY > Y_LIMIT) and isTarget() ){
			if( modeActive = 1 ){
				execSaver()
			}
			modeActive := 0
		}
		;ToolTip( "past=" past " pos=(" posX "," posY ")",,,1)
	} catch Error as err {
		;
	}
	Sleep(100)
}

ExitApp()

/* セーバーを起動する */
execSaver(){
	try {
		Run(saverPath " /s")
	} catch Error as err {
		MsgBox( "ScreenSaver not found", , "0x40010 T10")
		ExitApp()
	}
}

/* セーバー起動対象のウィンドウかどうかを判定 */
isTarget(){
	if(ExecForce){
		return 1
	}
	idx := 1
	while( Path.Length >= idx ){
		if WinActive("ahk_exe " Path[idx]){
			return 1
		}
		idx := idx + 1
	}
	return 0
}

/* メニューからの呼び出し */
handleMenuItem(ItemName, ItemPos, MyMenu){
	execAfterWait()
}

/* 手動で開始する */
execAfterWait(){
	MsgBox("スクリーンセーバーを５秒後に開始します",,"0x40040 T5")
	execSaver()
}

#HotIf enableKey
	;SHIFT+Z でセーバーの起動
	+Z::
	{
		execAfterWait()
	}
#HotIf

